from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.archive_record import ArchiveRecord
from ..types import UNSET, Unset

T = TypeVar("T", bound="Folder")


@attr.s(auto_attribs=True, repr=False)
class Folder:
    """  """

    _archive_record: Union[Unset, None, ArchiveRecord] = UNSET
    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _parent_folder_id: Union[Unset, None, str] = UNSET
    _project_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("archive_record={}".format(repr(self._archive_record)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("parent_folder_id={}".format(repr(self._parent_folder_id)))
        fields.append("project_id={}".format(repr(self._project_id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "Folder({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        archive_record: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._archive_record, Unset):
            archive_record = self._archive_record.to_dict() if self._archive_record else None

        id = self._id
        name = self._name
        parent_folder_id = self._parent_folder_id
        project_id = self._project_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if parent_folder_id is not UNSET:
            field_dict["parentFolderId"] = parent_folder_id
        if project_id is not UNSET:
            field_dict["projectId"] = project_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_archive_record() -> Union[Unset, None, ArchiveRecord]:
            archive_record = None
            _archive_record = d.pop("archiveRecord")

            if _archive_record is not None and not isinstance(_archive_record, Unset):
                archive_record = ArchiveRecord.from_dict(_archive_record)

            return archive_record

        try:
            archive_record = get_archive_record()
        except KeyError:
            if strict:
                raise
            archive_record = cast(Union[Unset, None, ArchiveRecord], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_parent_folder_id() -> Union[Unset, None, str]:
            parent_folder_id = d.pop("parentFolderId")
            return parent_folder_id

        try:
            parent_folder_id = get_parent_folder_id()
        except KeyError:
            if strict:
                raise
            parent_folder_id = cast(Union[Unset, None, str], UNSET)

        def get_project_id() -> Union[Unset, str]:
            project_id = d.pop("projectId")
            return project_id

        try:
            project_id = get_project_id()
        except KeyError:
            if strict:
                raise
            project_id = cast(Union[Unset, str], UNSET)

        folder = cls(
            archive_record=archive_record,
            id=id,
            name=name,
            parent_folder_id=parent_folder_id,
            project_id=project_id,
        )

        folder.additional_properties = d
        return folder

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def archive_record(self) -> Optional[ArchiveRecord]:
        if isinstance(self._archive_record, Unset):
            raise NotPresentError(self, "archive_record")
        return self._archive_record

    @archive_record.setter
    def archive_record(self, value: Optional[ArchiveRecord]) -> None:
        self._archive_record = value

    @archive_record.deleter
    def archive_record(self) -> None:
        self._archive_record = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def parent_folder_id(self) -> Optional[str]:
        """ ID of the parent folder, if it exists """
        if isinstance(self._parent_folder_id, Unset):
            raise NotPresentError(self, "parent_folder_id")
        return self._parent_folder_id

    @parent_folder_id.setter
    def parent_folder_id(self, value: Optional[str]) -> None:
        self._parent_folder_id = value

    @parent_folder_id.deleter
    def parent_folder_id(self) -> None:
        self._parent_folder_id = UNSET

    @property
    def project_id(self) -> str:
        """ ID of the containing project """
        if isinstance(self._project_id, Unset):
            raise NotPresentError(self, "project_id")
        return self._project_id

    @project_id.setter
    def project_id(self, value: str) -> None:
        self._project_id = value

    @project_id.deleter
    def project_id(self) -> None:
        self._project_id = UNSET



import warnings

import benchling_api_client.v2.benchling_client

AuthorizationMethod = benchling_api_client.v2.benchling_client.AuthorizationMethod
BenchlingApiClient = benchling_api_client.v2.benchling_client.BenchlingApiClient

warnings.warn(
    "This package is deprecated: please use benchling_api_client.v2.benchling_client", DeprecationWarning
)



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.workflow_task_execution_origin_type import WorkflowTaskExecutionOriginType
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskExecutionOrigin")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskExecutionOrigin:
    """ The context into which a task was executed """

    _entry_id: Union[Unset, None, str] = UNSET
    _origin_modal_uuid: Union[Unset, None, str] = UNSET
    _type: Union[Unset, WorkflowTaskExecutionOriginType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("entry_id={}".format(repr(self._entry_id)))
        fields.append("origin_modal_uuid={}".format(repr(self._origin_modal_uuid)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTaskExecutionOrigin({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        entry_id = self._entry_id
        origin_modal_uuid = self._origin_modal_uuid
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if entry_id is not UNSET:
            field_dict["entryId"] = entry_id
        if origin_modal_uuid is not UNSET:
            field_dict["originModalUuid"] = origin_modal_uuid
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_entry_id() -> Union[Unset, None, str]:
            entry_id = d.pop("entryId")
            return entry_id

        try:
            entry_id = get_entry_id()
        except KeyError:
            if strict:
                raise
            entry_id = cast(Union[Unset, None, str], UNSET)

        def get_origin_modal_uuid() -> Union[Unset, None, str]:
            origin_modal_uuid = d.pop("originModalUuid")
            return origin_modal_uuid

        try:
            origin_modal_uuid = get_origin_modal_uuid()
        except KeyError:
            if strict:
                raise
            origin_modal_uuid = cast(Union[Unset, None, str], UNSET)

        def get_type() -> Union[Unset, WorkflowTaskExecutionOriginType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = WorkflowTaskExecutionOriginType(_type)
                except ValueError:
                    type = WorkflowTaskExecutionOriginType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, WorkflowTaskExecutionOriginType], UNSET)

        workflow_task_execution_origin = cls(
            entry_id=entry_id,
            origin_modal_uuid=origin_modal_uuid,
            type=type,
        )

        workflow_task_execution_origin.additional_properties = d
        return workflow_task_execution_origin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def entry_id(self) -> Optional[str]:
        if isinstance(self._entry_id, Unset):
            raise NotPresentError(self, "entry_id")
        return self._entry_id

    @entry_id.setter
    def entry_id(self, value: Optional[str]) -> None:
        self._entry_id = value

    @entry_id.deleter
    def entry_id(self) -> None:
        self._entry_id = UNSET

    @property
    def origin_modal_uuid(self) -> Optional[str]:
        if isinstance(self._origin_modal_uuid, Unset):
            raise NotPresentError(self, "origin_modal_uuid")
        return self._origin_modal_uuid

    @origin_modal_uuid.setter
    def origin_modal_uuid(self, value: Optional[str]) -> None:
        self._origin_modal_uuid = value

    @origin_modal_uuid.deleter
    def origin_modal_uuid(self) -> None:
        self._origin_modal_uuid = UNSET

    @property
    def type(self) -> WorkflowTaskExecutionOriginType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: WorkflowTaskExecutionOriginType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="User")


@attr.s(auto_attribs=True, repr=False)
class User:
    """  """

    _email: Union[Unset, str] = UNSET
    _is_suspended: Union[Unset, bool] = UNSET
    _password_last_changed_at: Union[Unset, None, str] = UNSET
    _handle: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("email={}".format(repr(self._email)))
        fields.append("is_suspended={}".format(repr(self._is_suspended)))
        fields.append("password_last_changed_at={}".format(repr(self._password_last_changed_at)))
        fields.append("handle={}".format(repr(self._handle)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "User({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        email = self._email
        is_suspended = self._is_suspended
        password_last_changed_at = self._password_last_changed_at
        handle = self._handle
        id = self._id
        name = self._name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if email is not UNSET:
            field_dict["email"] = email
        if is_suspended is not UNSET:
            field_dict["isSuspended"] = is_suspended
        if password_last_changed_at is not UNSET:
            field_dict["passwordLastChangedAt"] = password_last_changed_at
        if handle is not UNSET:
            field_dict["handle"] = handle
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_email() -> Union[Unset, str]:
            email = d.pop("email")
            return email

        try:
            email = get_email()
        except KeyError:
            if strict:
                raise
            email = cast(Union[Unset, str], UNSET)

        def get_is_suspended() -> Union[Unset, bool]:
            is_suspended = d.pop("isSuspended")
            return is_suspended

        try:
            is_suspended = get_is_suspended()
        except KeyError:
            if strict:
                raise
            is_suspended = cast(Union[Unset, bool], UNSET)

        def get_password_last_changed_at() -> Union[Unset, None, str]:
            password_last_changed_at = d.pop("passwordLastChangedAt")
            return password_last_changed_at

        try:
            password_last_changed_at = get_password_last_changed_at()
        except KeyError:
            if strict:
                raise
            password_last_changed_at = cast(Union[Unset, None, str], UNSET)

        def get_handle() -> Union[Unset, str]:
            handle = d.pop("handle")
            return handle

        try:
            handle = get_handle()
        except KeyError:
            if strict:
                raise
            handle = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        user = cls(
            email=email,
            is_suspended=is_suspended,
            password_last_changed_at=password_last_changed_at,
            handle=handle,
            id=id,
            name=name,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def email(self) -> str:
        if isinstance(self._email, Unset):
            raise NotPresentError(self, "email")
        return self._email

    @email.setter
    def email(self, value: str) -> None:
        self._email = value

    @email.deleter
    def email(self) -> None:
        self._email = UNSET

    @property
    def is_suspended(self) -> bool:
        if isinstance(self._is_suspended, Unset):
            raise NotPresentError(self, "is_suspended")
        return self._is_suspended

    @is_suspended.setter
    def is_suspended(self, value: bool) -> None:
        self._is_suspended = value

    @is_suspended.deleter
    def is_suspended(self) -> None:
        self._is_suspended = UNSET

    @property
    def password_last_changed_at(self) -> Optional[str]:
        if isinstance(self._password_last_changed_at, Unset):
            raise NotPresentError(self, "password_last_changed_at")
        return self._password_last_changed_at

    @password_last_changed_at.setter
    def password_last_changed_at(self, value: Optional[str]) -> None:
        self._password_last_changed_at = value

    @password_last_changed_at.deleter
    def password_last_changed_at(self) -> None:
        self._password_last_changed_at = UNSET

    @property
    def handle(self) -> str:
        if isinstance(self._handle, Unset):
            raise NotPresentError(self, "handle")
        return self._handle

    @handle.setter
    def handle(self, value: str) -> None:
        self._handle = value

    @handle.deleter
    def handle(self) -> None:
        self._handle = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.benchling_app import BenchlingApp
from ..types import UNSET, Unset

T = TypeVar("T", bound="BenchlingAppsPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class BenchlingAppsPaginatedList:
    """  """

    _apps: Union[Unset, List[BenchlingApp]] = UNSET
    _next_token: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("apps={}".format(repr(self._apps)))
        fields.append("next_token={}".format(repr(self._next_token)))
        return "BenchlingAppsPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        apps: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._apps, Unset):
            apps = []
            for apps_item_data in self._apps:
                apps_item = apps_item_data.to_dict()

                apps.append(apps_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if apps is not UNSET:
            field_dict["apps"] = apps
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_apps() -> Union[Unset, List[BenchlingApp]]:
            apps = []
            _apps = d.pop("apps")
            for apps_item_data in _apps or []:
                apps_item = BenchlingApp.from_dict(apps_item_data, strict=False)

                apps.append(apps_item)

            return apps

        try:
            apps = get_apps()
        except KeyError:
            if strict:
                raise
            apps = cast(Union[Unset, List[BenchlingApp]], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        benchling_apps_paginated_list = cls(
            apps=apps,
            next_token=next_token,
        )

        return benchling_apps_paginated_list

    @property
    def apps(self) -> List[BenchlingApp]:
        if isinstance(self._apps, Unset):
            raise NotPresentError(self, "apps")
        return self._apps

    @apps.setter
    def apps(self, value: List[BenchlingApp]) -> None:
        self._apps = value

    @apps.deleter
    def apps(self) -> None:
        self._apps = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.archive_record import ArchiveRecord
from ..models.dropdown_field_definition import DropdownFieldDefinition
from ..models.float_field_definition import FloatFieldDefinition
from ..models.integer_field_definition import IntegerFieldDefinition
from ..models.schema_link_field_definition import SchemaLinkFieldDefinition
from ..models.simple_field_definition import SimpleFieldDefinition
from ..models.team_summary import TeamSummary
from ..models.workflow_output_schema import WorkflowOutputSchema
from ..models.workflow_task_schema_execution_type import WorkflowTaskSchemaExecutionType
from ..models.workflow_task_status_lifecycle import WorkflowTaskStatusLifecycle
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskSchema")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskSchema:
    """  """

    _execution_type: Union[Unset, WorkflowTaskSchemaExecutionType] = UNSET
    _can_set_assignee_on_task_creation: Union[Unset, bool] = UNSET
    _default_creation_folder_id: Union[Unset, None, str] = UNSET
    _default_entry_execution_folder_id: Union[Unset, None, str] = UNSET
    _default_responsible_team: Union[Unset, None, TeamSummary] = UNSET
    _entry_template_id: Union[Unset, None, str] = UNSET
    _prefix: Union[Unset, str] = UNSET
    _status_lifecycle: Union[Unset, WorkflowTaskStatusLifecycle] = UNSET
    _task_group_prefix: Union[Unset, str] = UNSET
    _workflow_output_schema: Union[Unset, None, WorkflowOutputSchema] = UNSET
    _archive_record: Union[Unset, None, ArchiveRecord] = UNSET
    _field_definitions: Union[
        Unset,
        List[
            Union[
                SimpleFieldDefinition,
                IntegerFieldDefinition,
                FloatFieldDefinition,
                DropdownFieldDefinition,
                SchemaLinkFieldDefinition,
                UnknownType,
            ]
        ],
    ] = UNSET
    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("execution_type={}".format(repr(self._execution_type)))
        fields.append(
            "can_set_assignee_on_task_creation={}".format(repr(self._can_set_assignee_on_task_creation))
        )
        fields.append("default_creation_folder_id={}".format(repr(self._default_creation_folder_id)))
        fields.append(
            "default_entry_execution_folder_id={}".format(repr(self._default_entry_execution_folder_id))
        )
        fields.append("default_responsible_team={}".format(repr(self._default_responsible_team)))
        fields.append("entry_template_id={}".format(repr(self._entry_template_id)))
        fields.append("prefix={}".format(repr(self._prefix)))
        fields.append("status_lifecycle={}".format(repr(self._status_lifecycle)))
        fields.append("task_group_prefix={}".format(repr(self._task_group_prefix)))
        fields.append("workflow_output_schema={}".format(repr(self._workflow_output_schema)))
        fields.append("archive_record={}".format(repr(self._archive_record)))
        fields.append("field_definitions={}".format(repr(self._field_definitions)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTaskSchema({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        execution_type: Union[Unset, int] = UNSET
        if not isinstance(self._execution_type, Unset):
            execution_type = self._execution_type.value

        can_set_assignee_on_task_creation = self._can_set_assignee_on_task_creation
        default_creation_folder_id = self._default_creation_folder_id
        default_entry_execution_folder_id = self._default_entry_execution_folder_id
        default_responsible_team: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._default_responsible_team, Unset):
            default_responsible_team = (
                self._default_responsible_team.to_dict() if self._default_responsible_team else None
            )

        entry_template_id = self._entry_template_id
        prefix = self._prefix
        status_lifecycle: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._status_lifecycle, Unset):
            status_lifecycle = self._status_lifecycle.to_dict()

        task_group_prefix = self._task_group_prefix
        workflow_output_schema: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._workflow_output_schema, Unset):
            workflow_output_schema = (
                self._workflow_output_schema.to_dict() if self._workflow_output_schema else None
            )

        archive_record: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._archive_record, Unset):
            archive_record = self._archive_record.to_dict() if self._archive_record else None

        field_definitions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._field_definitions, Unset):
            field_definitions = []
            for field_definitions_item_data in self._field_definitions:
                if isinstance(field_definitions_item_data, UnknownType):
                    field_definitions_item = field_definitions_item_data.value
                elif isinstance(field_definitions_item_data, SimpleFieldDefinition):
                    field_definitions_item = field_definitions_item_data.to_dict()

                elif isinstance(field_definitions_item_data, IntegerFieldDefinition):
                    field_definitions_item = field_definitions_item_data.to_dict()

                elif isinstance(field_definitions_item_data, FloatFieldDefinition):
                    field_definitions_item = field_definitions_item_data.to_dict()

                elif isinstance(field_definitions_item_data, DropdownFieldDefinition):
                    field_definitions_item = field_definitions_item_data.to_dict()

                else:
                    field_definitions_item = field_definitions_item_data.to_dict()

                field_definitions.append(field_definitions_item)

        id = self._id
        name = self._name
        type = self._type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if execution_type is not UNSET:
            field_dict["executionType"] = execution_type
        if can_set_assignee_on_task_creation is not UNSET:
            field_dict["canSetAssigneeOnTaskCreation"] = can_set_assignee_on_task_creation
        if default_creation_folder_id is not UNSET:
            field_dict["defaultCreationFolderId"] = default_creation_folder_id
        if default_entry_execution_folder_id is not UNSET:
            field_dict["defaultEntryExecutionFolderId"] = default_entry_execution_folder_id
        if default_responsible_team is not UNSET:
            field_dict["defaultResponsibleTeam"] = default_responsible_team
        if entry_template_id is not UNSET:
            field_dict["entryTemplateId"] = entry_template_id
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if status_lifecycle is not UNSET:
            field_dict["statusLifecycle"] = status_lifecycle
        if task_group_prefix is not UNSET:
            field_dict["taskGroupPrefix"] = task_group_prefix
        if workflow_output_schema is not UNSET:
            field_dict["workflowOutputSchema"] = workflow_output_schema
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if field_definitions is not UNSET:
            field_dict["fieldDefinitions"] = field_definitions
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_execution_type() -> Union[Unset, WorkflowTaskSchemaExecutionType]:
            execution_type = UNSET
            _execution_type = d.pop("executionType")
            if _execution_type is not None and _execution_type is not UNSET:
                try:
                    execution_type = WorkflowTaskSchemaExecutionType(_execution_type)
                except ValueError:
                    execution_type = WorkflowTaskSchemaExecutionType.of_unknown(_execution_type)

            return execution_type

        try:
            execution_type = get_execution_type()
        except KeyError:
            if strict:
                raise
            execution_type = cast(Union[Unset, WorkflowTaskSchemaExecutionType], UNSET)

        def get_can_set_assignee_on_task_creation() -> Union[Unset, bool]:
            can_set_assignee_on_task_creation = d.pop("canSetAssigneeOnTaskCreation")
            return can_set_assignee_on_task_creation

        try:
            can_set_assignee_on_task_creation = get_can_set_assignee_on_task_creation()
        except KeyError:
            if strict:
                raise
            can_set_assignee_on_task_creation = cast(Union[Unset, bool], UNSET)

        def get_default_creation_folder_id() -> Union[Unset, None, str]:
            default_creation_folder_id = d.pop("defaultCreationFolderId")
            return default_creation_folder_id

        try:
            default_creation_folder_id = get_default_creation_folder_id()
        except KeyError:
            if strict:
                raise
            default_creation_folder_id = cast(Union[Unset, None, str], UNSET)

        def get_default_entry_execution_folder_id() -> Union[Unset, None, str]:
            default_entry_execution_folder_id = d.pop("defaultEntryExecutionFolderId")
            return default_entry_execution_folder_id

        try:
            default_entry_execution_folder_id = get_default_entry_execution_folder_id()
        except KeyError:
            if strict:
                raise
            default_entry_execution_folder_id = cast(Union[Unset, None, str], UNSET)

        def get_default_responsible_team() -> Union[Unset, None, TeamSummary]:
            default_responsible_team = None
            _default_responsible_team = d.pop("defaultResponsibleTeam")

            if _default_responsible_team is not None and not isinstance(_default_responsible_team, Unset):
                default_responsible_team = TeamSummary.from_dict(_default_responsible_team)

            return default_responsible_team

        try:
            default_responsible_team = get_default_responsible_team()
        except KeyError:
            if strict:
                raise
            default_responsible_team = cast(Union[Unset, None, TeamSummary], UNSET)

        def get_entry_template_id() -> Union[Unset, None, str]:
            entry_template_id = d.pop("entryTemplateId")
            return entry_template_id

        try:
            entry_template_id = get_entry_template_id()
        except KeyError:
            if strict:
                raise
            entry_template_id = cast(Union[Unset, None, str], UNSET)

        def get_prefix() -> Union[Unset, str]:
            prefix = d.pop("prefix")
            return prefix

        try:
            prefix = get_prefix()
        except KeyError:
            if strict:
                raise
            prefix = cast(Union[Unset, str], UNSET)

        def get_status_lifecycle() -> Union[Unset, WorkflowTaskStatusLifecycle]:
            status_lifecycle: Union[Unset, Union[Unset, WorkflowTaskStatusLifecycle]] = UNSET
            _status_lifecycle = d.pop("statusLifecycle")

            if not isinstance(_status_lifecycle, Unset):
                status_lifecycle = WorkflowTaskStatusLifecycle.from_dict(_status_lifecycle)

            return status_lifecycle

        try:
            status_lifecycle = get_status_lifecycle()
        except KeyError:
            if strict:
                raise
            status_lifecycle = cast(Union[Unset, WorkflowTaskStatusLifecycle], UNSET)

        def get_task_group_prefix() -> Union[Unset, str]:
            task_group_prefix = d.pop("taskGroupPrefix")
            return task_group_prefix

        try:
            task_group_prefix = get_task_group_prefix()
        except KeyError:
            if strict:
                raise
            task_group_prefix = cast(Union[Unset, str], UNSET)

        def get_workflow_output_schema() -> Union[Unset, None, WorkflowOutputSchema]:
            workflow_output_schema = None
            _workflow_output_schema = d.pop("workflowOutputSchema")

            if _workflow_output_schema is not None and not isinstance(_workflow_output_schema, Unset):
                workflow_output_schema = WorkflowOutputSchema.from_dict(_workflow_output_schema)

            return workflow_output_schema

        try:
            workflow_output_schema = get_workflow_output_schema()
        except KeyError:
            if strict:
                raise
            workflow_output_schema = cast(Union[Unset, None, WorkflowOutputSchema], UNSET)

        def get_archive_record() -> Union[Unset, None, ArchiveRecord]:
            archive_record = None
            _archive_record = d.pop("archiveRecord")

            if _archive_record is not None and not isinstance(_archive_record, Unset):
                archive_record = ArchiveRecord.from_dict(_archive_record)

            return archive_record

        try:
            archive_record = get_archive_record()
        except KeyError:
            if strict:
                raise
            archive_record = cast(Union[Unset, None, ArchiveRecord], UNSET)

        def get_field_definitions() -> Union[
            Unset,
            List[
                Union[
                    SimpleFieldDefinition,
                    IntegerFieldDefinition,
                    FloatFieldDefinition,
                    DropdownFieldDefinition,
                    SchemaLinkFieldDefinition,
                    UnknownType,
                ]
            ],
        ]:
            field_definitions = []
            _field_definitions = d.pop("fieldDefinitions")
            for field_definitions_item_data in _field_definitions or []:

                def _parse_field_definitions_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    SimpleFieldDefinition,
                    IntegerFieldDefinition,
                    FloatFieldDefinition,
                    DropdownFieldDefinition,
                    SchemaLinkFieldDefinition,
                    UnknownType,
                ]:
                    field_definitions_item: Union[
                        SimpleFieldDefinition,
                        IntegerFieldDefinition,
                        FloatFieldDefinition,
                        DropdownFieldDefinition,
                        SchemaLinkFieldDefinition,
                        UnknownType,
                    ]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = SimpleFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = IntegerFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = FloatFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = DropdownFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = SchemaLinkFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                field_definitions_item = _parse_field_definitions_item(field_definitions_item_data)

                field_definitions.append(field_definitions_item)

            return field_definitions

        try:
            field_definitions = get_field_definitions()
        except KeyError:
            if strict:
                raise
            field_definitions = cast(
                Union[
                    Unset,
                    List[
                        Union[
                            SimpleFieldDefinition,
                            IntegerFieldDefinition,
                            FloatFieldDefinition,
                            DropdownFieldDefinition,
                            SchemaLinkFieldDefinition,
                            UnknownType,
                        ]
                    ],
                ],
                UNSET,
            )

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_type() -> Union[Unset, str]:
            type = d.pop("type")
            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, str], UNSET)

        workflow_task_schema = cls(
            execution_type=execution_type,
            can_set_assignee_on_task_creation=can_set_assignee_on_task_creation,
            default_creation_folder_id=default_creation_folder_id,
            default_entry_execution_folder_id=default_entry_execution_folder_id,
            default_responsible_team=default_responsible_team,
            entry_template_id=entry_template_id,
            prefix=prefix,
            status_lifecycle=status_lifecycle,
            task_group_prefix=task_group_prefix,
            workflow_output_schema=workflow_output_schema,
            archive_record=archive_record,
            field_definitions=field_definitions,
            id=id,
            name=name,
            type=type,
        )

        workflow_task_schema.additional_properties = d
        return workflow_task_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def execution_type(self) -> WorkflowTaskSchemaExecutionType:
        """ The method by which instances of this schema are executed """
        if isinstance(self._execution_type, Unset):
            raise NotPresentError(self, "execution_type")
        return self._execution_type

    @execution_type.setter
    def execution_type(self, value: WorkflowTaskSchemaExecutionType) -> None:
        self._execution_type = value

    @execution_type.deleter
    def execution_type(self) -> None:
        self._execution_type = UNSET

    @property
    def can_set_assignee_on_task_creation(self) -> bool:
        """ Whether or not tasks of this schema can be created with a non-null assignee. """
        if isinstance(self._can_set_assignee_on_task_creation, Unset):
            raise NotPresentError(self, "can_set_assignee_on_task_creation")
        return self._can_set_assignee_on_task_creation

    @can_set_assignee_on_task_creation.setter
    def can_set_assignee_on_task_creation(self, value: bool) -> None:
        self._can_set_assignee_on_task_creation = value

    @can_set_assignee_on_task_creation.deleter
    def can_set_assignee_on_task_creation(self) -> None:
        self._can_set_assignee_on_task_creation = UNSET

    @property
    def default_creation_folder_id(self) -> Optional[str]:
        """ ID of the default folder for creating workflow task groups """
        if isinstance(self._default_creation_folder_id, Unset):
            raise NotPresentError(self, "default_creation_folder_id")
        return self._default_creation_folder_id

    @default_creation_folder_id.setter
    def default_creation_folder_id(self, value: Optional[str]) -> None:
        self._default_creation_folder_id = value

    @default_creation_folder_id.deleter
    def default_creation_folder_id(self) -> None:
        self._default_creation_folder_id = UNSET

    @property
    def default_entry_execution_folder_id(self) -> Optional[str]:
        """ ID of the default folder for workflow task execution entries """
        if isinstance(self._default_entry_execution_folder_id, Unset):
            raise NotPresentError(self, "default_entry_execution_folder_id")
        return self._default_entry_execution_folder_id

    @default_entry_execution_folder_id.setter
    def default_entry_execution_folder_id(self, value: Optional[str]) -> None:
        self._default_entry_execution_folder_id = value

    @default_entry_execution_folder_id.deleter
    def default_entry_execution_folder_id(self) -> None:
        self._default_entry_execution_folder_id = UNSET

    @property
    def default_responsible_team(self) -> Optional[TeamSummary]:
        if isinstance(self._default_responsible_team, Unset):
            raise NotPresentError(self, "default_responsible_team")
        return self._default_responsible_team

    @default_responsible_team.setter
    def default_responsible_team(self, value: Optional[TeamSummary]) -> None:
        self._default_responsible_team = value

    @default_responsible_team.deleter
    def default_responsible_team(self) -> None:
        self._default_responsible_team = UNSET

    @property
    def entry_template_id(self) -> Optional[str]:
        """ The ID of the template of the entries tasks of this schema will be executed into. """
        if isinstance(self._entry_template_id, Unset):
            raise NotPresentError(self, "entry_template_id")
        return self._entry_template_id

    @entry_template_id.setter
    def entry_template_id(self, value: Optional[str]) -> None:
        self._entry_template_id = value

    @entry_template_id.deleter
    def entry_template_id(self) -> None:
        self._entry_template_id = UNSET

    @property
    def prefix(self) -> str:
        """ The prefix for the displayId of tasks of this schema. """
        if isinstance(self._prefix, Unset):
            raise NotPresentError(self, "prefix")
        return self._prefix

    @prefix.setter
    def prefix(self, value: str) -> None:
        self._prefix = value

    @prefix.deleter
    def prefix(self) -> None:
        self._prefix = UNSET

    @property
    def status_lifecycle(self) -> WorkflowTaskStatusLifecycle:
        if isinstance(self._status_lifecycle, Unset):
            raise NotPresentError(self, "status_lifecycle")
        return self._status_lifecycle

    @status_lifecycle.setter
    def status_lifecycle(self, value: WorkflowTaskStatusLifecycle) -> None:
        self._status_lifecycle = value

    @status_lifecycle.deleter
    def status_lifecycle(self) -> None:
        self._status_lifecycle = UNSET

    @property
    def task_group_prefix(self) -> str:
        """ The prefix for the displayId of task groups containing tasks of this schema """
        if isinstance(self._task_group_prefix, Unset):
            raise NotPresentError(self, "task_group_prefix")
        return self._task_group_prefix

    @task_group_prefix.setter
    def task_group_prefix(self, value: str) -> None:
        self._task_group_prefix = value

    @task_group_prefix.deleter
    def task_group_prefix(self) -> None:
        self._task_group_prefix = UNSET

    @property
    def workflow_output_schema(self) -> Optional[WorkflowOutputSchema]:
        if isinstance(self._workflow_output_schema, Unset):
            raise NotPresentError(self, "workflow_output_schema")
        return self._workflow_output_schema

    @workflow_output_schema.setter
    def workflow_output_schema(self, value: Optional[WorkflowOutputSchema]) -> None:
        self._workflow_output_schema = value

    @workflow_output_schema.deleter
    def workflow_output_schema(self) -> None:
        self._workflow_output_schema = UNSET

    @property
    def archive_record(self) -> Optional[ArchiveRecord]:
        if isinstance(self._archive_record, Unset):
            raise NotPresentError(self, "archive_record")
        return self._archive_record

    @archive_record.setter
    def archive_record(self, value: Optional[ArchiveRecord]) -> None:
        self._archive_record = value

    @archive_record.deleter
    def archive_record(self) -> None:
        self._archive_record = UNSET

    @property
    def field_definitions(
        self,
    ) -> List[
        Union[
            SimpleFieldDefinition,
            IntegerFieldDefinition,
            FloatFieldDefinition,
            DropdownFieldDefinition,
            SchemaLinkFieldDefinition,
            UnknownType,
        ]
    ]:
        if isinstance(self._field_definitions, Unset):
            raise NotPresentError(self, "field_definitions")
        return self._field_definitions

    @field_definitions.setter
    def field_definitions(
        self,
        value: List[
            Union[
                SimpleFieldDefinition,
                IntegerFieldDefinition,
                FloatFieldDefinition,
                DropdownFieldDefinition,
                SchemaLinkFieldDefinition,
                UnknownType,
            ]
        ],
    ) -> None:
        self._field_definitions = value

    @field_definitions.deleter
    def field_definitions(self) -> None:
        self._field_definitions = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def type(self) -> str:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: str) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.schema_class import SchemaClass
from ..types import UNSET, Unset

T = TypeVar("T", bound="SchemaClassesList")


@attr.s(auto_attribs=True, repr=False)
class SchemaClassesList:
    """  """

    _schema_classes: Union[Unset, List[SchemaClass]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("schema_classes={}".format(repr(self._schema_classes)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "SchemaClassesList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        schema_classes: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._schema_classes, Unset):
            schema_classes = []
            for schema_classes_item_data in self._schema_classes:
                schema_classes_item = schema_classes_item_data.to_dict()

                schema_classes.append(schema_classes_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if schema_classes is not UNSET:
            field_dict["schemaClasses"] = schema_classes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_schema_classes() -> Union[Unset, List[SchemaClass]]:
            schema_classes = []
            _schema_classes = d.pop("schemaClasses")
            for schema_classes_item_data in _schema_classes or []:
                schema_classes_item = SchemaClass.from_dict(schema_classes_item_data, strict=False)

                schema_classes.append(schema_classes_item)

            return schema_classes

        try:
            schema_classes = get_schema_classes()
        except KeyError:
            if strict:
                raise
            schema_classes = cast(Union[Unset, List[SchemaClass]], UNSET)

        schema_classes_list = cls(
            schema_classes=schema_classes,
        )

        schema_classes_list.additional_properties = d
        return schema_classes_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def schema_classes(self) -> List[SchemaClass]:
        if isinstance(self._schema_classes, Unset):
            raise NotPresentError(self, "schema_classes")
        return self._schema_classes

    @schema_classes.setter
    def schema_classes(self, value: List[SchemaClass]) -> None:
        self._schema_classes = value

    @schema_classes.deleter
    def schema_classes(self) -> None:
        self._schema_classes = UNSET



from typing import Dict

import attr
import httpx


@attr.s(auto_attribs=True)
class Client:
    """A class for keeping track of data related to the API."""

    base_url: str
    httpx_client: httpx.Client = attr.ib(factory=httpx.Client, kw_only=True)
    cookies: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    headers: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    timeout: float = attr.ib(5.0, kw_only=True)

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in all endpoints."""
        return {**self.headers}

    def with_headers(self, headers: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional headers."""
        return attr.evolve(self, headers={**self.headers, **headers})

    def get_cookies(self) -> Dict[str, str]:
        return {**self.cookies}

    def with_cookies(self, cookies: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional cookies."""
        return attr.evolve(self, cookies={**self.cookies, **cookies})

    def get_timeout(self) -> float:
        return self.timeout

    def with_timeout(self, timeout: float, increase_only: bool = True) -> "Client":
        """
        Get a new client matching this one with a new timeout (in seconds).

        :param timeout: The new client timeout in seconds
        :param increase_only: If True, will only use the specified timeout if it's
        higher than the existing Client timeout
        :return: A new Client with the adjusted timeout
        :rtype: Client
        """
        if (not increase_only) or timeout > self.timeout:
            return attr.evolve(self, timeout=timeout)
        return self

    def with_base_url(self, base_url: str) -> "Client":
        """Get a new client matching this one with an updated base URL."""
        return attr.evolve(self, base_url=base_url)


@attr.s(auto_attribs=True)
class AuthenticatedClient(Client):
    """A Client which has been authenticated for use on secured endpoints."""

    token: str

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in authenticated endpoints."""
        return {"Authorization": f"Bearer {self.token}", **self.headers}



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.container_labels import ContainerLabels
from ..models.entity_labels import EntityLabels
from ..models.ingredient_component_entity import IngredientComponentEntity
from ..models.ingredient_measurement_units import IngredientMeasurementUnits
from ..types import UNSET, Unset

T = TypeVar("T", bound="Ingredient")


@attr.s(auto_attribs=True, repr=False)
class Ingredient:
    """  """

    _amount: Union[Unset, None, str] = UNSET
    _catalog_identifier: Union[Unset, None, str] = UNSET
    _component_entity: Union[Unset, IngredientComponentEntity] = UNSET
    _component_lot_container: Union[Unset, None, ContainerLabels] = UNSET
    _component_lot_entity: Union[Unset, None, EntityLabels] = UNSET
    _component_lot_text: Union[Unset, None, str] = UNSET
    _has_parent: Union[Unset, bool] = UNSET
    _notes: Union[Unset, None, str] = UNSET
    _target_amount: Union[Unset, None, str] = UNSET
    _units: Union[Unset, IngredientMeasurementUnits] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("amount={}".format(repr(self._amount)))
        fields.append("catalog_identifier={}".format(repr(self._catalog_identifier)))
        fields.append("component_entity={}".format(repr(self._component_entity)))
        fields.append("component_lot_container={}".format(repr(self._component_lot_container)))
        fields.append("component_lot_entity={}".format(repr(self._component_lot_entity)))
        fields.append("component_lot_text={}".format(repr(self._component_lot_text)))
        fields.append("has_parent={}".format(repr(self._has_parent)))
        fields.append("notes={}".format(repr(self._notes)))
        fields.append("target_amount={}".format(repr(self._target_amount)))
        fields.append("units={}".format(repr(self._units)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "Ingredient({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        amount = self._amount
        catalog_identifier = self._catalog_identifier
        component_entity: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._component_entity, Unset):
            component_entity = self._component_entity.to_dict()

        component_lot_container: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._component_lot_container, Unset):
            component_lot_container = (
                self._component_lot_container.to_dict() if self._component_lot_container else None
            )

        component_lot_entity: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._component_lot_entity, Unset):
            component_lot_entity = (
                self._component_lot_entity.to_dict() if self._component_lot_entity else None
            )

        component_lot_text = self._component_lot_text
        has_parent = self._has_parent
        notes = self._notes
        target_amount = self._target_amount
        units: Union[Unset, int] = UNSET
        if not isinstance(self._units, Unset):
            units = self._units.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if amount is not UNSET:
            field_dict["amount"] = amount
        if catalog_identifier is not UNSET:
            field_dict["catalogIdentifier"] = catalog_identifier
        if component_entity is not UNSET:
            field_dict["componentEntity"] = component_entity
        if component_lot_container is not UNSET:
            field_dict["componentLotContainer"] = component_lot_container
        if component_lot_entity is not UNSET:
            field_dict["componentLotEntity"] = component_lot_entity
        if component_lot_text is not UNSET:
            field_dict["componentLotText"] = component_lot_text
        if has_parent is not UNSET:
            field_dict["hasParent"] = has_parent
        if notes is not UNSET:
            field_dict["notes"] = notes
        if target_amount is not UNSET:
            field_dict["targetAmount"] = target_amount
        if units is not UNSET:
            field_dict["units"] = units

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_amount() -> Union[Unset, None, str]:
            amount = d.pop("amount")
            return amount

        try:
            amount = get_amount()
        except KeyError:
            if strict:
                raise
            amount = cast(Union[Unset, None, str], UNSET)

        def get_catalog_identifier() -> Union[Unset, None, str]:
            catalog_identifier = d.pop("catalogIdentifier")
            return catalog_identifier

        try:
            catalog_identifier = get_catalog_identifier()
        except KeyError:
            if strict:
                raise
            catalog_identifier = cast(Union[Unset, None, str], UNSET)

        def get_component_entity() -> Union[Unset, IngredientComponentEntity]:
            component_entity: Union[Unset, Union[Unset, IngredientComponentEntity]] = UNSET
            _component_entity = d.pop("componentEntity")

            if not isinstance(_component_entity, Unset):
                component_entity = IngredientComponentEntity.from_dict(_component_entity)

            return component_entity

        try:
            component_entity = get_component_entity()
        except KeyError:
            if strict:
                raise
            component_entity = cast(Union[Unset, IngredientComponentEntity], UNSET)

        def get_component_lot_container() -> Union[Unset, None, ContainerLabels]:
            component_lot_container = None
            _component_lot_container = d.pop("componentLotContainer")

            if _component_lot_container is not None and not isinstance(_component_lot_container, Unset):
                component_lot_container = ContainerLabels.from_dict(_component_lot_container)

            return component_lot_container

        try:
            component_lot_container = get_component_lot_container()
        except KeyError:
            if strict:
                raise
            component_lot_container = cast(Union[Unset, None, ContainerLabels], UNSET)

        def get_component_lot_entity() -> Union[Unset, None, EntityLabels]:
            component_lot_entity = None
            _component_lot_entity = d.pop("componentLotEntity")

            if _component_lot_entity is not None and not isinstance(_component_lot_entity, Unset):
                component_lot_entity = EntityLabels.from_dict(_component_lot_entity)

            return component_lot_entity

        try:
            component_lot_entity = get_component_lot_entity()
        except KeyError:
            if strict:
                raise
            component_lot_entity = cast(Union[Unset, None, EntityLabels], UNSET)

        def get_component_lot_text() -> Union[Unset, None, str]:
            component_lot_text = d.pop("componentLotText")
            return component_lot_text

        try:
            component_lot_text = get_component_lot_text()
        except KeyError:
            if strict:
                raise
            component_lot_text = cast(Union[Unset, None, str], UNSET)

        def get_has_parent() -> Union[Unset, bool]:
            has_parent = d.pop("hasParent")
            return has_parent

        try:
            has_parent = get_has_parent()
        except KeyError:
            if strict:
                raise
            has_parent = cast(Union[Unset, bool], UNSET)

        def get_notes() -> Union[Unset, None, str]:
            notes = d.pop("notes")
            return notes

        try:
            notes = get_notes()
        except KeyError:
            if strict:
                raise
            notes = cast(Union[Unset, None, str], UNSET)

        def get_target_amount() -> Union[Unset, None, str]:
            target_amount = d.pop("targetAmount")
            return target_amount

        try:
            target_amount = get_target_amount()
        except KeyError:
            if strict:
                raise
            target_amount = cast(Union[Unset, None, str], UNSET)

        def get_units() -> Union[Unset, IngredientMeasurementUnits]:
            units = UNSET
            _units = d.pop("units")
            if _units is not None and _units is not UNSET:
                try:
                    units = IngredientMeasurementUnits(_units)
                except ValueError:
                    units = IngredientMeasurementUnits.of_unknown(_units)

            return units

        try:
            units = get_units()
        except KeyError:
            if strict:
                raise
            units = cast(Union[Unset, IngredientMeasurementUnits], UNSET)

        ingredient = cls(
            amount=amount,
            catalog_identifier=catalog_identifier,
            component_entity=component_entity,
            component_lot_container=component_lot_container,
            component_lot_entity=component_lot_entity,
            component_lot_text=component_lot_text,
            has_parent=has_parent,
            notes=notes,
            target_amount=target_amount,
            units=units,
        )

        ingredient.additional_properties = d
        return ingredient

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def amount(self) -> Optional[str]:
        """The amount value of this ingredient in its mixture, in string format (to preserve full precision). Pair with `units`. Supports scientific notation (1.23e4). One ingredient on this mixture can have an amount value of `"QS"`."""
        if isinstance(self._amount, Unset):
            raise NotPresentError(self, "amount")
        return self._amount

    @amount.setter
    def amount(self, value: Optional[str]) -> None:
        self._amount = value

    @amount.deleter
    def amount(self) -> None:
        self._amount = UNSET

    @property
    def catalog_identifier(self) -> Optional[str]:
        if isinstance(self._catalog_identifier, Unset):
            raise NotPresentError(self, "catalog_identifier")
        return self._catalog_identifier

    @catalog_identifier.setter
    def catalog_identifier(self, value: Optional[str]) -> None:
        self._catalog_identifier = value

    @catalog_identifier.deleter
    def catalog_identifier(self) -> None:
        self._catalog_identifier = UNSET

    @property
    def component_entity(self) -> IngredientComponentEntity:
        if isinstance(self._component_entity, Unset):
            raise NotPresentError(self, "component_entity")
        return self._component_entity

    @component_entity.setter
    def component_entity(self, value: IngredientComponentEntity) -> None:
        self._component_entity = value

    @component_entity.deleter
    def component_entity(self) -> None:
        self._component_entity = UNSET

    @property
    def component_lot_container(self) -> Optional[ContainerLabels]:
        if isinstance(self._component_lot_container, Unset):
            raise NotPresentError(self, "component_lot_container")
        return self._component_lot_container

    @component_lot_container.setter
    def component_lot_container(self, value: Optional[ContainerLabels]) -> None:
        self._component_lot_container = value

    @component_lot_container.deleter
    def component_lot_container(self) -> None:
        self._component_lot_container = UNSET

    @property
    def component_lot_entity(self) -> Optional[EntityLabels]:
        if isinstance(self._component_lot_entity, Unset):
            raise NotPresentError(self, "component_lot_entity")
        return self._component_lot_entity

    @component_lot_entity.setter
    def component_lot_entity(self, value: Optional[EntityLabels]) -> None:
        self._component_lot_entity = value

    @component_lot_entity.deleter
    def component_lot_entity(self) -> None:
        self._component_lot_entity = UNSET

    @property
    def component_lot_text(self) -> Optional[str]:
        """ Text representing the component lot for this ingredient. This is only present if the mixture schema supports component lots in "text" format. """
        if isinstance(self._component_lot_text, Unset):
            raise NotPresentError(self, "component_lot_text")
        return self._component_lot_text

    @component_lot_text.setter
    def component_lot_text(self, value: Optional[str]) -> None:
        self._component_lot_text = value

    @component_lot_text.deleter
    def component_lot_text(self) -> None:
        self._component_lot_text = UNSET

    @property
    def has_parent(self) -> bool:
        if isinstance(self._has_parent, Unset):
            raise NotPresentError(self, "has_parent")
        return self._has_parent

    @has_parent.setter
    def has_parent(self, value: bool) -> None:
        self._has_parent = value

    @has_parent.deleter
    def has_parent(self) -> None:
        self._has_parent = UNSET

    @property
    def notes(self) -> Optional[str]:
        if isinstance(self._notes, Unset):
            raise NotPresentError(self, "notes")
        return self._notes

    @notes.setter
    def notes(self, value: Optional[str]) -> None:
        self._notes = value

    @notes.deleter
    def notes(self) -> None:
        self._notes = UNSET

    @property
    def target_amount(self) -> Optional[str]:
        """ The target amount for this ingredient such that this ingredient's proportion in its mixture would preserve the equivalent ingredient's proportion in the parent mixture. Pair with `units`. """
        if isinstance(self._target_amount, Unset):
            raise NotPresentError(self, "target_amount")
        return self._target_amount

    @target_amount.setter
    def target_amount(self, value: Optional[str]) -> None:
        self._target_amount = value

    @target_amount.deleter
    def target_amount(self) -> None:
        self._target_amount = UNSET

    @property
    def units(self) -> IngredientMeasurementUnits:
        if isinstance(self._units, Unset):
            raise NotPresentError(self, "units")
        return self._units

    @units.setter
    def units(self, value: IngredientMeasurementUnits) -> None:
        self._units = value

    @units.deleter
    def units(self) -> None:
        self._units = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.lab_automation_benchling_app_errors_top_level_errors_item import (
    LabAutomationBenchlingAppErrorsTopLevelErrorsItem,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="LabAutomationBenchlingAppErrors")


@attr.s(auto_attribs=True, repr=False)
class LabAutomationBenchlingAppErrors:
    """  """

    _top_level_errors: Union[Unset, List[LabAutomationBenchlingAppErrorsTopLevelErrorsItem]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("top_level_errors={}".format(repr(self._top_level_errors)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "LabAutomationBenchlingAppErrors({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        top_level_errors: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._top_level_errors, Unset):
            top_level_errors = []
            for top_level_errors_item_data in self._top_level_errors:
                top_level_errors_item = top_level_errors_item_data.to_dict()

                top_level_errors.append(top_level_errors_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if top_level_errors is not UNSET:
            field_dict["topLevelErrors"] = top_level_errors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_top_level_errors() -> Union[Unset, List[LabAutomationBenchlingAppErrorsTopLevelErrorsItem]]:
            top_level_errors = []
            _top_level_errors = d.pop("topLevelErrors")
            for top_level_errors_item_data in _top_level_errors or []:
                top_level_errors_item = LabAutomationBenchlingAppErrorsTopLevelErrorsItem.from_dict(
                    top_level_errors_item_data, strict=False
                )

                top_level_errors.append(top_level_errors_item)

            return top_level_errors

        try:
            top_level_errors = get_top_level_errors()
        except KeyError:
            if strict:
                raise
            top_level_errors = cast(
                Union[Unset, List[LabAutomationBenchlingAppErrorsTopLevelErrorsItem]], UNSET
            )

        lab_automation_benchling_app_errors = cls(
            top_level_errors=top_level_errors,
        )

        lab_automation_benchling_app_errors.additional_properties = d
        return lab_automation_benchling_app_errors

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def top_level_errors(self) -> List[LabAutomationBenchlingAppErrorsTopLevelErrorsItem]:
        if isinstance(self._top_level_errors, Unset):
            raise NotPresentError(self, "top_level_errors")
        return self._top_level_errors

    @top_level_errors.setter
    def top_level_errors(self, value: List[LabAutomationBenchlingAppErrorsTopLevelErrorsItem]) -> None:
        self._top_level_errors = value

    @top_level_errors.deleter
    def top_level_errors(self) -> None:
        self._top_level_errors = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.custom_fields import CustomFields
from ..models.fields import Fields
from ..models.initial_table import InitialTable
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryCreateAttachments")


@attr.s(auto_attribs=True, repr=False)
class EntryCreateAttachments:
    """  """

    _folder_id: str
    _name: str
    _initial_tables: Union[Unset, List[InitialTable]] = UNSET
    _attachment_text: Union[Unset, None] = UNSET
    _attachments: Union[Unset, List[str]] = UNSET
    _author_ids: Union[Unset, str, List[str], UnknownType] = UNSET
    _custom_fields: Union[Unset, CustomFields] = UNSET
    _entry_template_id: Union[Unset, str] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _schema_id: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("folder_id={}".format(repr(self._folder_id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("initial_tables={}".format(repr(self._initial_tables)))
        fields.append("attachment_text={}".format(repr(self._attachment_text)))
        fields.append("attachments={}".format(repr(self._attachments)))
        fields.append("author_ids={}".format(repr(self._author_ids)))
        fields.append("custom_fields={}".format(repr(self._custom_fields)))
        fields.append("entry_template_id={}".format(repr(self._entry_template_id)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("schema_id={}".format(repr(self._schema_id)))
        return "EntryCreateAttachments({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        folder_id = self._folder_id
        name = self._name
        initial_tables: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._initial_tables, Unset):
            initial_tables = []
            for initial_tables_item_data in self._initial_tables:
                initial_tables_item = initial_tables_item_data.to_dict()

                initial_tables.append(initial_tables_item)

        attachment_text = None

        attachments: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._attachments, Unset):
            attachments = self._attachments

        author_ids: Union[Unset, str, List[Any]]
        if isinstance(self._author_ids, Unset):
            author_ids = UNSET
        elif isinstance(self._author_ids, UnknownType):
            author_ids = self._author_ids.value
        elif isinstance(self._author_ids, list):
            author_ids = UNSET
            if not isinstance(self._author_ids, Unset):
                author_ids = self._author_ids

        else:
            author_ids = self._author_ids

        custom_fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._custom_fields, Unset):
            custom_fields = self._custom_fields.to_dict()

        entry_template_id = self._entry_template_id
        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        schema_id = self._schema_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if name is not UNSET:
            field_dict["name"] = name
        if initial_tables is not UNSET:
            field_dict["initialTables"] = initial_tables
        if attachment_text is not UNSET:
            field_dict["attachmentText"] = attachment_text
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if author_ids is not UNSET:
            field_dict["authorIds"] = author_ids
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if entry_template_id is not UNSET:
            field_dict["entryTemplateId"] = entry_template_id
        if fields is not UNSET:
            field_dict["fields"] = fields
        if schema_id is not UNSET:
            field_dict["schemaId"] = schema_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_folder_id() -> str:
            folder_id = d.pop("folderId")
            return folder_id

        try:
            folder_id = get_folder_id()
        except KeyError:
            if strict:
                raise
            folder_id = cast(str, UNSET)

        def get_name() -> str:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(str, UNSET)

        def get_initial_tables() -> Union[Unset, List[InitialTable]]:
            initial_tables = []
            _initial_tables = d.pop("initialTables")
            for initial_tables_item_data in _initial_tables or []:
                initial_tables_item = InitialTable.from_dict(initial_tables_item_data, strict=False)

                initial_tables.append(initial_tables_item)

            return initial_tables

        try:
            initial_tables = get_initial_tables()
        except KeyError:
            if strict:
                raise
            initial_tables = cast(Union[Unset, List[InitialTable]], UNSET)

        def get_attachment_text() -> Union[Unset, None]:
            attachment_text = None

            return attachment_text

        try:
            attachment_text = get_attachment_text()
        except KeyError:
            if strict:
                raise
            attachment_text = cast(Union[Unset, None], UNSET)

        def get_attachments() -> Union[Unset, List[str]]:
            attachments = cast(List[str], d.pop("attachments"))

            return attachments

        try:
            attachments = get_attachments()
        except KeyError:
            if strict:
                raise
            attachments = cast(Union[Unset, List[str]], UNSET)

        def get_author_ids() -> Union[Unset, str, List[str], UnknownType]:
            def _parse_author_ids(
                data: Union[Unset, str, List[Any]]
            ) -> Union[Unset, str, List[str], UnknownType]:
                author_ids: Union[Unset, str, List[str], UnknownType]
                if isinstance(data, Unset):
                    return data
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    author_ids = cast(List[str], data)

                    return author_ids
                except:  # noqa: E722
                    pass
                if isinstance(data, dict):
                    return UnknownType(data)
                return cast(Union[Unset, str, List[str], UnknownType], data)

            author_ids = _parse_author_ids(d.pop("authorIds"))

            return author_ids

        try:
            author_ids = get_author_ids()
        except KeyError:
            if strict:
                raise
            author_ids = cast(Union[Unset, str, List[str], UnknownType], UNSET)

        def get_custom_fields() -> Union[Unset, CustomFields]:
            custom_fields: Union[Unset, Union[Unset, CustomFields]] = UNSET
            _custom_fields = d.pop("customFields")

            if not isinstance(_custom_fields, Unset):
                custom_fields = CustomFields.from_dict(_custom_fields)

            return custom_fields

        try:
            custom_fields = get_custom_fields()
        except KeyError:
            if strict:
                raise
            custom_fields = cast(Union[Unset, CustomFields], UNSET)

        def get_entry_template_id() -> Union[Unset, str]:
            entry_template_id = d.pop("entryTemplateId")
            return entry_template_id

        try:
            entry_template_id = get_entry_template_id()
        except KeyError:
            if strict:
                raise
            entry_template_id = cast(Union[Unset, str], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_schema_id() -> Union[Unset, str]:
            schema_id = d.pop("schemaId")
            return schema_id

        try:
            schema_id = get_schema_id()
        except KeyError:
            if strict:
                raise
            schema_id = cast(Union[Unset, str], UNSET)

        entry_create_attachments = cls(
            folder_id=folder_id,
            name=name,
            initial_tables=initial_tables,
            attachment_text=attachment_text,
            attachments=attachments,
            author_ids=author_ids,
            custom_fields=custom_fields,
            entry_template_id=entry_template_id,
            fields=fields,
            schema_id=schema_id,
        )

        return entry_create_attachments

    @property
    def folder_id(self) -> str:
        """ ID of the folder that will contain the entry """
        if isinstance(self._folder_id, Unset):
            raise NotPresentError(self, "folder_id")
        return self._folder_id

    @folder_id.setter
    def folder_id(self, value: str) -> None:
        self._folder_id = value

    @property
    def name(self) -> str:
        """ Name of the entry """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @property
    def initial_tables(self) -> List[InitialTable]:
        """An array of table API IDs and blob id pairs to seed tables from the template while creating the entry. The entryTemplateId parameter must be set to use this parameter. The table API IDs should be the API Identifiers of the tables in the given template.
        - If a template table has one row, the values in that row act as default values for cloned entries.
        - If a template table has multiple rows, there is no default value and those rows are added to the cloned entry along with the provided csv data.
        - If a table has default values, they will be populated in any respective undefined columns in the csv data.
        - If a table has no default values, undefined columns from csv data will be empty.
        - If no csv data is provided for a table, the table in the entry will be populated with whatever values are in the respective template table.
        """
        if isinstance(self._initial_tables, Unset):
            raise NotPresentError(self, "initial_tables")
        return self._initial_tables

    @initial_tables.setter
    def initial_tables(self, value: List[InitialTable]) -> None:
        self._initial_tables = value

    @initial_tables.deleter
    def initial_tables(self) -> None:
        self._initial_tables = UNSET

    @property
    def attachment_text(self) -> None:
        """Text to include in the entry before the attachments. If not provided, the text will be "File attachments are listed below:" """
        if isinstance(self._attachment_text, Unset):
            raise NotPresentError(self, "attachment_text")
        return self._attachment_text

    @attachment_text.setter
    def attachment_text(self, value: None) -> None:
        self._attachment_text = value

    @attachment_text.deleter
    def attachment_text(self) -> None:
        self._attachment_text = UNSET

    @property
    def attachments(self) -> List[str]:
        """An array of IDs representing Blob objects in Benchling to be attached to the entry."""
        if isinstance(self._attachments, Unset):
            raise NotPresentError(self, "attachments")
        return self._attachments

    @attachments.setter
    def attachments(self, value: List[str]) -> None:
        self._attachments = value

    @attachments.deleter
    def attachments(self) -> None:
        self._attachments = UNSET

    @property
    def author_ids(self) -> Union[str, List[str], UnknownType]:
        if isinstance(self._author_ids, Unset):
            raise NotPresentError(self, "author_ids")
        return self._author_ids

    @author_ids.setter
    def author_ids(self, value: Union[str, List[str], UnknownType]) -> None:
        self._author_ids = value

    @author_ids.deleter
    def author_ids(self) -> None:
        self._author_ids = UNSET

    @property
    def custom_fields(self) -> CustomFields:
        if isinstance(self._custom_fields, Unset):
            raise NotPresentError(self, "custom_fields")
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, value: CustomFields) -> None:
        self._custom_fields = value

    @custom_fields.deleter
    def custom_fields(self) -> None:
        self._custom_fields = UNSET

    @property
    def entry_template_id(self) -> str:
        """ ID of the template to clone the entry from """
        if isinstance(self._entry_template_id, Unset):
            raise NotPresentError(self, "entry_template_id")
        return self._entry_template_id

    @entry_template_id.setter
    def entry_template_id(self, value: str) -> None:
        self._entry_template_id = value

    @entry_template_id.deleter
    def entry_template_id(self) -> None:
        self._entry_template_id = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def schema_id(self) -> str:
        """ ID of the entry's schema """
        if isinstance(self._schema_id, Unset):
            raise NotPresentError(self, "schema_id")
        return self._schema_id

    @schema_id.setter
    def schema_id(self, value: str) -> None:
        self._schema_id = value

    @schema_id.deleter
    def schema_id(self) -> None:
        self._schema_id = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.member_collaborator import MemberCollaborator
from ..models.principal_collaborator import PrincipalCollaborator

CollaborationCreate = Union[PrincipalCollaborator, MemberCollaborator, UnknownType]



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorklistUpdate")


@attr.s(auto_attribs=True, repr=False)
class WorklistUpdate:
    """  """

    _name: Union[Unset, str] = UNSET
    _worklist_item_ids: Union[Unset, List[str]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("name={}".format(repr(self._name)))
        fields.append("worklist_item_ids={}".format(repr(self._worklist_item_ids)))
        return "WorklistUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        name = self._name
        worklist_item_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._worklist_item_ids, Unset):
            worklist_item_ids = self._worklist_item_ids

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if name is not UNSET:
            field_dict["name"] = name
        if worklist_item_ids is not UNSET:
            field_dict["worklistItemIds"] = worklist_item_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_worklist_item_ids() -> Union[Unset, List[str]]:
            worklist_item_ids = cast(List[str], d.pop("worklistItemIds"))

            return worklist_item_ids

        try:
            worklist_item_ids = get_worklist_item_ids()
        except KeyError:
            if strict:
                raise
            worklist_item_ids = cast(Union[Unset, List[str]], UNSET)

        worklist_update = cls(
            name=name,
            worklist_item_ids=worklist_item_ids,
        )

        return worklist_update

    @property
    def name(self) -> str:
        """ Name of the worklist """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def worklist_item_ids(self) -> List[str]:
        """An ordered set of IDs to assign as worklist items. IDs should reference existing items which fit the worklist's specific type. For instance, a worklist of type container should only have item IDs which represent containers.
        Replaces any existing worklist items with this set.
        """
        if isinstance(self._worklist_item_ids, Unset):
            raise NotPresentError(self, "worklist_item_ids")
        return self._worklist_item_ids

    @worklist_item_ids.setter
    def worklist_item_ids(self, value: List[str]) -> None:
        self._worklist_item_ids = value

    @worklist_item_ids.deleter
    def worklist_item_ids(self) -> None:
        self._worklist_item_ids = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="BarcodeValidationResult")


@attr.s(auto_attribs=True, repr=False)
class BarcodeValidationResult:
    """  """

    _barcode: Union[Unset, str] = UNSET
    _is_valid: Union[Unset, bool] = UNSET
    _message: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("barcode={}".format(repr(self._barcode)))
        fields.append("is_valid={}".format(repr(self._is_valid)))
        fields.append("message={}".format(repr(self._message)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BarcodeValidationResult({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        barcode = self._barcode
        is_valid = self._is_valid
        message = self._message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if barcode is not UNSET:
            field_dict["barcode"] = barcode
        if is_valid is not UNSET:
            field_dict["isValid"] = is_valid
        if message is not UNSET:
            field_dict["message"] = message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_barcode() -> Union[Unset, str]:
            barcode = d.pop("barcode")
            return barcode

        try:
            barcode = get_barcode()
        except KeyError:
            if strict:
                raise
            barcode = cast(Union[Unset, str], UNSET)

        def get_is_valid() -> Union[Unset, bool]:
            is_valid = d.pop("isValid")
            return is_valid

        try:
            is_valid = get_is_valid()
        except KeyError:
            if strict:
                raise
            is_valid = cast(Union[Unset, bool], UNSET)

        def get_message() -> Union[Unset, None, str]:
            message = d.pop("message")
            return message

        try:
            message = get_message()
        except KeyError:
            if strict:
                raise
            message = cast(Union[Unset, None, str], UNSET)

        barcode_validation_result = cls(
            barcode=barcode,
            is_valid=is_valid,
            message=message,
        )

        barcode_validation_result.additional_properties = d
        return barcode_validation_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def barcode(self) -> str:
        """ Barcode to validate. """
        if isinstance(self._barcode, Unset):
            raise NotPresentError(self, "barcode")
        return self._barcode

    @barcode.setter
    def barcode(self, value: str) -> None:
        self._barcode = value

    @barcode.deleter
    def barcode(self) -> None:
        self._barcode = UNSET

    @property
    def is_valid(self) -> bool:
        """ Whether the barcode is valid. """
        if isinstance(self._is_valid, Unset):
            raise NotPresentError(self, "is_valid")
        return self._is_valid

    @is_valid.setter
    def is_valid(self, value: bool) -> None:
        self._is_valid = value

    @is_valid.deleter
    def is_valid(self) -> None:
        self._is_valid = UNSET

    @property
    def message(self) -> Optional[str]:
        """ If barcode is not valid, a message string explaining the error. """
        if isinstance(self._message, Unset):
            raise NotPresentError(self, "message")
        return self._message

    @message.setter
    def message(self, value: Optional[str]) -> None:
        self._message = value

    @message.deleter
    def message(self) -> None:
        self._message = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.workflow_flowchart_edge_config import WorkflowFlowchartEdgeConfig
from ..models.workflow_flowchart_node_config import WorkflowFlowchartNodeConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowFlowchart")


@attr.s(auto_attribs=True, repr=False)
class WorkflowFlowchart:
    """  """

    _edge_configs: Union[Unset, List[WorkflowFlowchartEdgeConfig]] = UNSET
    _id: Union[Unset, str] = UNSET
    _node_configs: Union[Unset, List[WorkflowFlowchartNodeConfig]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("edge_configs={}".format(repr(self._edge_configs)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("node_configs={}".format(repr(self._node_configs)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowFlowchart({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        edge_configs: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._edge_configs, Unset):
            edge_configs = []
            for edge_configs_item_data in self._edge_configs:
                edge_configs_item = edge_configs_item_data.to_dict()

                edge_configs.append(edge_configs_item)

        id = self._id
        node_configs: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._node_configs, Unset):
            node_configs = []
            for node_configs_item_data in self._node_configs:
                node_configs_item = node_configs_item_data.to_dict()

                node_configs.append(node_configs_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if edge_configs is not UNSET:
            field_dict["edgeConfigs"] = edge_configs
        if id is not UNSET:
            field_dict["id"] = id
        if node_configs is not UNSET:
            field_dict["nodeConfigs"] = node_configs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_edge_configs() -> Union[Unset, List[WorkflowFlowchartEdgeConfig]]:
            edge_configs = []
            _edge_configs = d.pop("edgeConfigs")
            for edge_configs_item_data in _edge_configs or []:
                edge_configs_item = WorkflowFlowchartEdgeConfig.from_dict(
                    edge_configs_item_data, strict=False
                )

                edge_configs.append(edge_configs_item)

            return edge_configs

        try:
            edge_configs = get_edge_configs()
        except KeyError:
            if strict:
                raise
            edge_configs = cast(Union[Unset, List[WorkflowFlowchartEdgeConfig]], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_node_configs() -> Union[Unset, List[WorkflowFlowchartNodeConfig]]:
            node_configs = []
            _node_configs = d.pop("nodeConfigs")
            for node_configs_item_data in _node_configs or []:
                node_configs_item = WorkflowFlowchartNodeConfig.from_dict(
                    node_configs_item_data, strict=False
                )

                node_configs.append(node_configs_item)

            return node_configs

        try:
            node_configs = get_node_configs()
        except KeyError:
            if strict:
                raise
            node_configs = cast(Union[Unset, List[WorkflowFlowchartNodeConfig]], UNSET)

        workflow_flowchart = cls(
            edge_configs=edge_configs,
            id=id,
            node_configs=node_configs,
        )

        workflow_flowchart.additional_properties = d
        return workflow_flowchart

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def edge_configs(self) -> List[WorkflowFlowchartEdgeConfig]:
        """ Edge configs in this flowchart """
        if isinstance(self._edge_configs, Unset):
            raise NotPresentError(self, "edge_configs")
        return self._edge_configs

    @edge_configs.setter
    def edge_configs(self, value: List[WorkflowFlowchartEdgeConfig]) -> None:
        self._edge_configs = value

    @edge_configs.deleter
    def edge_configs(self) -> None:
        self._edge_configs = UNSET

    @property
    def id(self) -> str:
        """ The ID of the flowchart """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def node_configs(self) -> List[WorkflowFlowchartNodeConfig]:
        """ Node configs in this flowchart """
        if isinstance(self._node_configs, Unset):
            raise NotPresentError(self, "node_configs")
        return self._node_configs

    @node_configs.setter
    def node_configs(self, value: List[WorkflowFlowchartNodeConfig]) -> None:
        self._node_configs = value

    @node_configs.deleter
    def node_configs(self) -> None:
        self._node_configs = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="BenchlingAppManifestSecurity")


@attr.s(auto_attribs=True, repr=False)
class BenchlingAppManifestSecurity:
    """  """

    _public_key: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("public_key={}".format(repr(self._public_key)))
        return "BenchlingAppManifestSecurity({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        public_key = self._public_key

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if public_key is not UNSET:
            field_dict["publicKey"] = public_key

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_public_key() -> Union[Unset, str]:
            public_key = d.pop("publicKey")
            return public_key

        try:
            public_key = get_public_key()
        except KeyError:
            if strict:
                raise
            public_key = cast(Union[Unset, str], UNSET)

        benchling_app_manifest_security = cls(
            public_key=public_key,
        )

        return benchling_app_manifest_security

    @property
    def public_key(self) -> str:
        """Public key used to encrypt secure_text values. The value is constrained:
        * value must be a public key PEM certificate
        * key type (kty) must be RSA
        * algorithm must be RSAES_OAEP_SHA_256
        * key size must be 2048 bits, with exponent=65537
        * key usage (use) must not be "signing". It can be either unspecified, or "encrypt/decrypt" """
        if isinstance(self._public_key, Unset):
            raise NotPresentError(self, "public_key")
        return self._public_key

    @public_key.setter
    def public_key(self, value: str) -> None:
        self._public_key = value

    @public_key.deleter
    def public_key(self) -> None:
        self._public_key = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AppSignalBase")


@attr.s(auto_attribs=True, repr=False)
class AppSignalBase:
    """  """

    _deprecated: bool
    _excluded_properties: List[str]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("deprecated={}".format(repr(self._deprecated)))
        fields.append("excluded_properties={}".format(repr(self._excluded_properties)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AppSignalBase({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        deprecated = self._deprecated
        excluded_properties = self._excluded_properties

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if deprecated is not UNSET:
            field_dict["deprecated"] = deprecated
        if excluded_properties is not UNSET:
            field_dict["excludedProperties"] = excluded_properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_deprecated() -> bool:
            deprecated = d.pop("deprecated")
            return deprecated

        try:
            deprecated = get_deprecated()
        except KeyError:
            if strict:
                raise
            deprecated = cast(bool, UNSET)

        def get_excluded_properties() -> List[str]:
            excluded_properties = cast(List[str], d.pop("excludedProperties"))

            return excluded_properties

        try:
            excluded_properties = get_excluded_properties()
        except KeyError:
            if strict:
                raise
            excluded_properties = cast(List[str], UNSET)

        app_signal_base = cls(
            deprecated=deprecated,
            excluded_properties=excluded_properties,
        )

        app_signal_base.additional_properties = d
        return app_signal_base

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def deprecated(self) -> bool:
        if isinstance(self._deprecated, Unset):
            raise NotPresentError(self, "deprecated")
        return self._deprecated

    @deprecated.setter
    def deprecated(self, value: bool) -> None:
        self._deprecated = value

    @property
    def excluded_properties(self) -> List[str]:
        """These properties have been dropped from the payload due to size."""
        if isinstance(self._excluded_properties, Unset):
            raise NotPresentError(self, "excluded_properties")
        return self._excluded_properties

    @excluded_properties.setter
    def excluded_properties(self, value: List[str]) -> None:
        self._excluded_properties = value



from typing import Union

from ..extensions import UnknownType
from ..models.analysis_step_dataset_data import AnalysisStepDatasetData
from ..models.analysis_step_file_data import AnalysisStepFileData

AnalysisStepData = Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType]



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExportsAsyncTaskResponse")


@attr.s(auto_attribs=True, repr=False)
class ExportsAsyncTaskResponse:
    """  """

    _download_url: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("download_url={}".format(repr(self._download_url)))
        return "ExportsAsyncTaskResponse({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        download_url = self._download_url

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if download_url is not UNSET:
            field_dict["downloadURL"] = download_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_download_url() -> Union[Unset, str]:
            download_url = d.pop("downloadURL")
            return download_url

        try:
            download_url = get_download_url()
        except KeyError:
            if strict:
                raise
            download_url = cast(Union[Unset, str], UNSET)

        exports_async_task_response = cls(
            download_url=download_url,
        )

        return exports_async_task_response

    @property
    def download_url(self) -> str:
        if isinstance(self._download_url, Unset):
            raise NotPresentError(self, "download_url")
        return self._download_url

    @download_url.setter
    def download_url(self, value: str) -> None:
        self._download_url = value

    @download_url.deleter
    def download_url(self) -> None:
        self._download_url = UNSET



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.bad_request_error import BadRequestError
from ...models.benchling_apps_paginated_list import BenchlingAppsPaginatedList
from ...models.forbidden_error import ForbiddenError
from ...models.list_benchling_apps_sort import ListBenchlingAppsSort
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListBenchlingAppsSort] = ListBenchlingAppsSort.MODIFIEDAT,
    ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    member_of: Union[Unset, str] = UNSET,
    admin_of: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/apps".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_sort: Union[Unset, int] = UNSET
    if not isinstance(sort, Unset):
        json_sort = sort.value

    params: Dict[str, Any] = {}
    if not isinstance(page_size, Unset) and page_size is not None:
        params["pageSize"] = page_size
    if not isinstance(next_token, Unset) and next_token is not None:
        params["nextToken"] = next_token
    if not isinstance(json_sort, Unset) and json_sort is not None:
        params["sort"] = json_sort
    if not isinstance(ids, Unset) and ids is not None:
        params["ids"] = ids
    if not isinstance(modified_at, Unset) and modified_at is not None:
        params["modifiedAt"] = modified_at
    if not isinstance(name, Unset) and name is not None:
        params["name"] = name
    if not isinstance(name_includes, Unset) and name_includes is not None:
        params["nameIncludes"] = name_includes
    if not isinstance(namesany_of, Unset) and namesany_of is not None:
        params["names.anyOf"] = namesany_of
    if not isinstance(namesany_ofcase_sensitive, Unset) and namesany_ofcase_sensitive is not None:
        params["names.anyOf.caseSensitive"] = namesany_ofcase_sensitive
    if not isinstance(creator_ids, Unset) and creator_ids is not None:
        params["creatorIds"] = creator_ids
    if not isinstance(member_of, Unset) and member_of is not None:
        params["memberOf"] = member_of
    if not isinstance(admin_of, Unset) and admin_of is not None:
        params["adminOf"] = admin_of

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[BenchlingAppsPaginatedList, BadRequestError, ForbiddenError]]:
    if response.status_code == 200:
        response_200 = BenchlingAppsPaginatedList.from_dict(response.json(), strict=False)

        return response_200
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    if response.status_code == 403:
        response_403 = ForbiddenError.from_dict(response.json(), strict=False)

        return response_403
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[BenchlingAppsPaginatedList, BadRequestError, ForbiddenError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListBenchlingAppsSort] = ListBenchlingAppsSort.MODIFIEDAT,
    ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    member_of: Union[Unset, str] = UNSET,
    admin_of: Union[Unset, str] = UNSET,
) -> Response[Union[BenchlingAppsPaginatedList, BadRequestError, ForbiddenError]]:
    kwargs = _get_kwargs(
        client=client,
        page_size=page_size,
        next_token=next_token,
        sort=sort,
        ids=ids,
        modified_at=modified_at,
        name=name,
        name_includes=name_includes,
        namesany_of=namesany_of,
        namesany_ofcase_sensitive=namesany_ofcase_sensitive,
        creator_ids=creator_ids,
        member_of=member_of,
        admin_of=admin_of,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListBenchlingAppsSort] = ListBenchlingAppsSort.MODIFIEDAT,
    ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    member_of: Union[Unset, str] = UNSET,
    admin_of: Union[Unset, str] = UNSET,
) -> Optional[Union[BenchlingAppsPaginatedList, BadRequestError, ForbiddenError]]:
    """List apps. Apps admin endpoints are moving to stable. Visit the [v2 docs](https://benchling.com/api/reference?showLA=true#/Apps)."""

    return sync_detailed(
        client=client,
        page_size=page_size,
        next_token=next_token,
        sort=sort,
        ids=ids,
        modified_at=modified_at,
        name=name,
        name_includes=name_includes,
        namesany_of=namesany_of,
        namesany_ofcase_sensitive=namesany_ofcase_sensitive,
        creator_ids=creator_ids,
        member_of=member_of,
        admin_of=admin_of,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListBenchlingAppsSort] = ListBenchlingAppsSort.MODIFIEDAT,
    ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    member_of: Union[Unset, str] = UNSET,
    admin_of: Union[Unset, str] = UNSET,
) -> Response[Union[BenchlingAppsPaginatedList, BadRequestError, ForbiddenError]]:
    kwargs = _get_kwargs(
        client=client,
        page_size=page_size,
        next_token=next_token,
        sort=sort,
        ids=ids,
        modified_at=modified_at,
        name=name,
        name_includes=name_includes,
        namesany_of=namesany_of,
        namesany_ofcase_sensitive=namesany_ofcase_sensitive,
        creator_ids=creator_ids,
        member_of=member_of,
        admin_of=admin_of,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListBenchlingAppsSort] = ListBenchlingAppsSort.MODIFIEDAT,
    ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    member_of: Union[Unset, str] = UNSET,
    admin_of: Union[Unset, str] = UNSET,
) -> Optional[Union[BenchlingAppsPaginatedList, BadRequestError, ForbiddenError]]:
    """List apps. Apps admin endpoints are moving to stable. Visit the [v2 docs](https://benchling.com/api/reference?showLA=true#/Apps)."""

    return (
        await asyncio_detailed(
            client=client,
            page_size=page_size,
            next_token=next_token,
            sort=sort,
            ids=ids,
            modified_at=modified_at,
            name=name,
            name_includes=name_includes,
            namesany_of=namesany_of,
            namesany_ofcase_sensitive=namesany_ofcase_sensitive,
            creator_ids=creator_ids,
            member_of=member_of,
            admin_of=admin_of,
        )
    ).parsed



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.assay_fields_create import AssayFieldsCreate
from ..models.assay_result_create_field_validation import AssayResultCreateFieldValidation
from ..models.fields import Fields
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssayResultCreate")


@attr.s(auto_attribs=True, repr=False)
class AssayResultCreate:
    """  """

    _fields: Union[Fields, AssayFieldsCreate, UnknownType]
    _schema_id: str
    _field_validation: Union[Unset, AssayResultCreateFieldValidation] = UNSET
    _id: Union[Unset, str] = UNSET
    _project_id: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("schema_id={}".format(repr(self._schema_id)))
        fields.append("field_validation={}".format(repr(self._field_validation)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("project_id={}".format(repr(self._project_id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AssayResultCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        if isinstance(self._fields, UnknownType):
            fields = self._fields.value
        elif isinstance(self._fields, Fields):
            fields = self._fields.to_dict()

        else:
            fields = self._fields.to_dict()

        schema_id = self._schema_id
        field_validation: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._field_validation, Unset):
            field_validation = self._field_validation.to_dict()

        id = self._id
        project_id = self._project_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if fields is not UNSET:
            field_dict["fields"] = fields
        if schema_id is not UNSET:
            field_dict["schemaId"] = schema_id
        if field_validation is not UNSET:
            field_dict["fieldValidation"] = field_validation
        if id is not UNSET:
            field_dict["id"] = id
        if project_id is not UNSET:
            field_dict["projectId"] = project_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_fields() -> Union[Fields, AssayFieldsCreate, UnknownType]:
            def _parse_fields(data: Union[Dict[str, Any]]) -> Union[Fields, AssayFieldsCreate, UnknownType]:
                fields: Union[Fields, AssayFieldsCreate, UnknownType]
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    fields = Fields.from_dict(data, strict=True)

                    return fields
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    fields = AssayFieldsCreate.from_dict(data, strict=True)

                    return fields
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            fields = _parse_fields(d.pop("fields"))

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Fields, AssayFieldsCreate, UnknownType], UNSET)

        def get_schema_id() -> str:
            schema_id = d.pop("schemaId")
            return schema_id

        try:
            schema_id = get_schema_id()
        except KeyError:
            if strict:
                raise
            schema_id = cast(str, UNSET)

        def get_field_validation() -> Union[Unset, AssayResultCreateFieldValidation]:
            field_validation: Union[Unset, Union[Unset, AssayResultCreateFieldValidation]] = UNSET
            _field_validation = d.pop("fieldValidation")

            if not isinstance(_field_validation, Unset):
                field_validation = AssayResultCreateFieldValidation.from_dict(_field_validation)

            return field_validation

        try:
            field_validation = get_field_validation()
        except KeyError:
            if strict:
                raise
            field_validation = cast(Union[Unset, AssayResultCreateFieldValidation], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_project_id() -> Union[Unset, None, str]:
            project_id = d.pop("projectId")
            return project_id

        try:
            project_id = get_project_id()
        except KeyError:
            if strict:
                raise
            project_id = cast(Union[Unset, None, str], UNSET)

        assay_result_create = cls(
            fields=fields,
            schema_id=schema_id,
            field_validation=field_validation,
            id=id,
            project_id=project_id,
        )

        assay_result_create.additional_properties = d
        return assay_result_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def fields(self) -> Union[Fields, AssayFieldsCreate, UnknownType]:
        """ Dictionary of result fields """
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Union[Fields, AssayFieldsCreate, UnknownType]) -> None:
        self._fields = value

    @property
    def schema_id(self) -> str:
        """ ID of result schema under which to upload this result """
        if isinstance(self._schema_id, Unset):
            raise NotPresentError(self, "schema_id")
        return self._schema_id

    @schema_id.setter
    def schema_id(self, value: str) -> None:
        self._schema_id = value

    @property
    def field_validation(self) -> AssayResultCreateFieldValidation:
        """Dictionary mapping field names to UserValidation Resources."""
        if isinstance(self._field_validation, Unset):
            raise NotPresentError(self, "field_validation")
        return self._field_validation

    @field_validation.setter
    def field_validation(self, value: AssayResultCreateFieldValidation) -> None:
        self._field_validation = value

    @field_validation.deleter
    def field_validation(self) -> None:
        self._field_validation = UNSET

    @property
    def id(self) -> str:
        """ UUID """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def project_id(self) -> Optional[str]:
        """The project that the assay result should be uploaded to. Only users with read access to the project will be able to read the assay result. Leaving this empty will result in only the creator having read access."""
        if isinstance(self._project_id, Unset):
            raise NotPresentError(self, "project_id")
        return self._project_id

    @project_id.setter
    def project_id(self, value: Optional[str]) -> None:
        self._project_id = value

    @project_id.deleter
    def project_id(self) -> None:
        self._project_id = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="BarcodesList")


@attr.s(auto_attribs=True, repr=False)
class BarcodesList:
    """  """

    _barcodes: List[str]

    def __repr__(self):
        fields = []
        fields.append("barcodes={}".format(repr(self._barcodes)))
        return "BarcodesList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        barcodes = self._barcodes

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if barcodes is not UNSET:
            field_dict["barcodes"] = barcodes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_barcodes() -> List[str]:
            barcodes = cast(List[str], d.pop("barcodes"))

            return barcodes

        try:
            barcodes = get_barcodes()
        except KeyError:
            if strict:
                raise
            barcodes = cast(List[str], UNSET)

        barcodes_list = cls(
            barcodes=barcodes,
        )

        return barcodes_list

    @property
    def barcodes(self) -> List[str]:
        """ Array of barcodes to validate. """
        if isinstance(self._barcodes, Unset):
            raise NotPresentError(self, "barcodes")
        return self._barcodes

    @barcodes.setter
    def barcodes(self, value: List[str]) -> None:
        self._barcodes = value



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.sample_group_status_update import SampleGroupStatusUpdate
from ..types import UNSET, Unset

T = TypeVar("T", bound="SampleGroupsStatusUpdate")


@attr.s(auto_attribs=True, repr=False)
class SampleGroupsStatusUpdate:
    """ Specification to update status of sample groups on the request which were executed. """

    _sample_groups: List[SampleGroupStatusUpdate]

    def __repr__(self):
        fields = []
        fields.append("sample_groups={}".format(repr(self._sample_groups)))
        return "SampleGroupsStatusUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        sample_groups = []
        for sample_groups_item_data in self._sample_groups:
            sample_groups_item = sample_groups_item_data.to_dict()

            sample_groups.append(sample_groups_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if sample_groups is not UNSET:
            field_dict["sampleGroups"] = sample_groups

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_sample_groups() -> List[SampleGroupStatusUpdate]:
            sample_groups = []
            _sample_groups = d.pop("sampleGroups")
            for sample_groups_item_data in _sample_groups:
                sample_groups_item = SampleGroupStatusUpdate.from_dict(sample_groups_item_data, strict=False)

                sample_groups.append(sample_groups_item)

            return sample_groups

        try:
            sample_groups = get_sample_groups()
        except KeyError:
            if strict:
                raise
            sample_groups = cast(List[SampleGroupStatusUpdate], UNSET)

        sample_groups_status_update = cls(
            sample_groups=sample_groups,
        )

        return sample_groups_status_update

    @property
    def sample_groups(self) -> List[SampleGroupStatusUpdate]:
        if isinstance(self._sample_groups, Unset):
            raise NotPresentError(self, "sample_groups")
        return self._sample_groups

    @sample_groups.setter
    def sample_groups(self, value: List[SampleGroupStatusUpdate]) -> None:
        self._sample_groups = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.new_blank_form_instance_provider_type import NewBlankFormInstanceProviderType
from ..types import UNSET, Unset

T = TypeVar("T", bound="NewBlankFormInstanceProvider")


@attr.s(auto_attribs=True, repr=False)
class NewBlankFormInstanceProvider:
    """  """

    _type: Union[Unset, NewBlankFormInstanceProviderType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "NewBlankFormInstanceProvider({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_type() -> Union[Unset, NewBlankFormInstanceProviderType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = NewBlankFormInstanceProviderType(_type)
                except ValueError:
                    type = NewBlankFormInstanceProviderType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, NewBlankFormInstanceProviderType], UNSET)

        new_blank_form_instance_provider = cls(
            type=type,
        )

        new_blank_form_instance_provider.additional_properties = d
        return new_blank_form_instance_provider

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def type(self) -> NewBlankFormInstanceProviderType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: NewBlankFormInstanceProviderType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.containers_archive_reason import ContainersArchiveReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainersArchive")


@attr.s(auto_attribs=True, repr=False)
class ContainersArchive:
    """  """

    _container_ids: List[str]
    _reason: ContainersArchiveReason
    _should_remove_barcodes: Union[Unset, bool] = UNSET

    def __repr__(self):
        fields = []
        fields.append("container_ids={}".format(repr(self._container_ids)))
        fields.append("reason={}".format(repr(self._reason)))
        fields.append("should_remove_barcodes={}".format(repr(self._should_remove_barcodes)))
        return "ContainersArchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        container_ids = self._container_ids

        reason = self._reason.value

        should_remove_barcodes = self._should_remove_barcodes

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if container_ids is not UNSET:
            field_dict["containerIds"] = container_ids
        if reason is not UNSET:
            field_dict["reason"] = reason
        if should_remove_barcodes is not UNSET:
            field_dict["shouldRemoveBarcodes"] = should_remove_barcodes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_container_ids() -> List[str]:
            container_ids = cast(List[str], d.pop("containerIds"))

            return container_ids

        try:
            container_ids = get_container_ids()
        except KeyError:
            if strict:
                raise
            container_ids = cast(List[str], UNSET)

        def get_reason() -> ContainersArchiveReason:
            _reason = d.pop("reason")
            try:
                reason = ContainersArchiveReason(_reason)
            except ValueError:
                reason = ContainersArchiveReason.of_unknown(_reason)

            return reason

        try:
            reason = get_reason()
        except KeyError:
            if strict:
                raise
            reason = cast(ContainersArchiveReason, UNSET)

        def get_should_remove_barcodes() -> Union[Unset, bool]:
            should_remove_barcodes = d.pop("shouldRemoveBarcodes")
            return should_remove_barcodes

        try:
            should_remove_barcodes = get_should_remove_barcodes()
        except KeyError:
            if strict:
                raise
            should_remove_barcodes = cast(Union[Unset, bool], UNSET)

        containers_archive = cls(
            container_ids=container_ids,
            reason=reason,
            should_remove_barcodes=should_remove_barcodes,
        )

        return containers_archive

    @property
    def container_ids(self) -> List[str]:
        """ Array of container IDs """
        if isinstance(self._container_ids, Unset):
            raise NotPresentError(self, "container_ids")
        return self._container_ids

    @container_ids.setter
    def container_ids(self, value: List[str]) -> None:
        self._container_ids = value

    @property
    def reason(self) -> ContainersArchiveReason:
        """Reason that containers are being archived."""
        if isinstance(self._reason, Unset):
            raise NotPresentError(self, "reason")
        return self._reason

    @reason.setter
    def reason(self, value: ContainersArchiveReason) -> None:
        self._reason = value

    @property
    def should_remove_barcodes(self) -> bool:
        """Remove barcodes. Removing barcodes from archived inventory that contain items will also remove barcodes from the contained items."""
        if isinstance(self._should_remove_barcodes, Unset):
            raise NotPresentError(self, "should_remove_barcodes")
        return self._should_remove_barcodes

    @should_remove_barcodes.setter
    def should_remove_barcodes(self, value: bool) -> None:
        self._should_remove_barcodes = value

    @should_remove_barcodes.deleter
    def should_remove_barcodes(self) -> None:
        self._should_remove_barcodes = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.assay_result_create import AssayResultCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssayResultsBulkCreateRequest")


@attr.s(auto_attribs=True, repr=False)
class AssayResultsBulkCreateRequest:
    """  """

    _assay_results: List[AssayResultCreate]

    def __repr__(self):
        fields = []
        fields.append("assay_results={}".format(repr(self._assay_results)))
        return "AssayResultsBulkCreateRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        assay_results = []
        for assay_results_item_data in self._assay_results:
            assay_results_item = assay_results_item_data.to_dict()

            assay_results.append(assay_results_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if assay_results is not UNSET:
            field_dict["assayResults"] = assay_results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_assay_results() -> List[AssayResultCreate]:
            assay_results = []
            _assay_results = d.pop("assayResults")
            for assay_results_item_data in _assay_results:
                assay_results_item = AssayResultCreate.from_dict(assay_results_item_data, strict=False)

                assay_results.append(assay_results_item)

            return assay_results

        try:
            assay_results = get_assay_results()
        except KeyError:
            if strict:
                raise
            assay_results = cast(List[AssayResultCreate], UNSET)

        assay_results_bulk_create_request = cls(
            assay_results=assay_results,
        )

        return assay_results_bulk_create_request

    @property
    def assay_results(self) -> List[AssayResultCreate]:
        if isinstance(self._assay_results, Unset):
            raise NotPresentError(self, "assay_results")
        return self._assay_results

    @assay_results.setter
    def assay_results(self, value: List[AssayResultCreate]) -> None:
        self._assay_results = value
